import { ChildProcessWithoutNullStreams } from 'node:child_process';

/**
 * BEL (Bell) is used to signal an alert.
 */
declare const BEL = "\u0007";
/**
 * BS (Backspace) moves the cursor one position to the left.
 */
declare const BS = "\b";
/**
 * HT (Horizontal Tab) moves the cursor one position to the right.
 */
declare const TAB = "\t";
declare const LF = "\n";
declare const VT = "\v";
declare const FF = "\f";
declare const CR = "\r";
declare const ESC = "\u001B";
declare const DEL = "";
declare const SPACE = " ";
declare const CURSOR_UP = "\u001B[A";
declare const CURSOR_DOWN = "\u001B[B";
declare const CURSOR_FORWARD = "\u001B[C";
declare const CURSOR_BACK = "\u001B[D";
declare const CURSOR_NEXT_LINE = "\u001B[E";
declare const CURSOR_PREVIOUS_LINE = "\u001B[F";
declare const CURSOR_HORIZONTAL_ABSOLUTE = "\u001B[G";
declare const CURSOR_HOME = "\u001B[H";
declare const ENTER = "\r";
declare const BACKSPACE = "\b";
declare const ARROW_UP = "\u001B[A";
declare const ARROW_DOWN = "\u001B[B";
declare const ARROW_RIGHT = "\u001B[C";
declare const ARROW_LEFT = "\u001B[D";

declare const ansi_ARROW_DOWN: typeof ARROW_DOWN;
declare const ansi_ARROW_LEFT: typeof ARROW_LEFT;
declare const ansi_ARROW_RIGHT: typeof ARROW_RIGHT;
declare const ansi_ARROW_UP: typeof ARROW_UP;
declare const ansi_BACKSPACE: typeof BACKSPACE;
declare const ansi_BEL: typeof BEL;
declare const ansi_BS: typeof BS;
declare const ansi_CR: typeof CR;
declare const ansi_CURSOR_BACK: typeof CURSOR_BACK;
declare const ansi_CURSOR_DOWN: typeof CURSOR_DOWN;
declare const ansi_CURSOR_FORWARD: typeof CURSOR_FORWARD;
declare const ansi_CURSOR_HOME: typeof CURSOR_HOME;
declare const ansi_CURSOR_HORIZONTAL_ABSOLUTE: typeof CURSOR_HORIZONTAL_ABSOLUTE;
declare const ansi_CURSOR_NEXT_LINE: typeof CURSOR_NEXT_LINE;
declare const ansi_CURSOR_PREVIOUS_LINE: typeof CURSOR_PREVIOUS_LINE;
declare const ansi_CURSOR_UP: typeof CURSOR_UP;
declare const ansi_DEL: typeof DEL;
declare const ansi_ENTER: typeof ENTER;
declare const ansi_ESC: typeof ESC;
declare const ansi_FF: typeof FF;
declare const ansi_LF: typeof LF;
declare const ansi_SPACE: typeof SPACE;
declare const ansi_TAB: typeof TAB;
declare const ansi_VT: typeof VT;
declare namespace ansi {
  export { ansi_ARROW_DOWN as ARROW_DOWN, ansi_ARROW_LEFT as ARROW_LEFT, ansi_ARROW_RIGHT as ARROW_RIGHT, ansi_ARROW_UP as ARROW_UP, ansi_BACKSPACE as BACKSPACE, ansi_BEL as BEL, ansi_BS as BS, ansi_CR as CR, ansi_CURSOR_BACK as CURSOR_BACK, ansi_CURSOR_DOWN as CURSOR_DOWN, ansi_CURSOR_FORWARD as CURSOR_FORWARD, ansi_CURSOR_HOME as CURSOR_HOME, ansi_CURSOR_HORIZONTAL_ABSOLUTE as CURSOR_HORIZONTAL_ABSOLUTE, ansi_CURSOR_NEXT_LINE as CURSOR_NEXT_LINE, ansi_CURSOR_PREVIOUS_LINE as CURSOR_PREVIOUS_LINE, ansi_CURSOR_UP as CURSOR_UP, ansi_DEL as DEL, ansi_ENTER as ENTER, ansi_ESC as ESC, ansi_FF as FF, ansi_LF as LF, ansi_SPACE as SPACE, ansi_TAB as TAB, ansi_VT as VT };
}

/**
 * Additonal options that can be passed to the CLITest constructor.
 */
type CLITestOptions = {
    /**
     * Throw an error if the process writes to stderr.
     * @default true
     */
    failOnStderr?: boolean;
    /**
     * The encoding to use for decoding the output.
     * @default 'utf-8'
     */
    outputEncoding?: BufferEncoding;
    process?: {
        /**
         * Current working directory of the process to be spawned.
         */
        cwd?: string;
        /**
         * Pass environment variables to the cli process.
         * @default process.env
         */
        env?: NodeJS.ProcessEnv;
        /**
         * Sets the user identity of the process.
         */
        uid?: number;
        /**
         * Sets the group identity of the process.
         */
        gid?: number;
        /**
         * Tte maximum amount of time the process is allowed to run in milliseconds.
         * @default undefined
         */
        timeout?: number;
    };
};
declare class CLITest {
    private command;
    private args;
    private childProcess;
    private running;
    private starting;
    private eventEmitter;
    private output;
    private options;
    /**
     * Create a new CLI test instance.
     * If you want to measure the code coverage of a JS/TS CLI, you may want to prepend the node command with 'c8' or 'nyc'.
     * @param command The command that should be executed to start the CLI process.
     * @param args The arguments that should be passed.
     * @param options Additional options.
     */
    constructor(command: string, args?: string[], options?: CLITestOptions);
    /**
     * Execute the CLI process.
     * Tip: You can call this method multiple times to restart the process, after the process has exited.
     * @throws If the process is already running or starting or if the command produces an error.
     * @returns A promise that resolves when the process has been started.
     */
    run(): Promise<void>;
    /**
     * Wait for the process to exit.
     * If the process returns a non-zero exit code, the promise will not be rejected.
     * @throws If the process is not running or starting.
     * @returns A promise that resolves when the process exits with the exit code.
     */
    waitForExit(): Promise<number | null>;
    /**
     * Wait for the process to write the given content to stdout or stderr.
     * If the process exits before the content is found, the promise will be rejected.
     * This method does not check past output, only output that is written after this method is called. For checking past output, use getOutput().
     * Per default this method is only able to find content that is written in a single chunk by the CLI process. If the content is split into multiple chunks set multipleChunks to true.
     *
     * @param content The string to wait for.
     * @param multipleChunks If true, the content can be split into multiple chunks. If false, the content must be written in a single chunk. Defaults to false.
     * @returns A promise that resolves when the content is found, or rejects if the process exits before the content is found.
     */
    waitForOutput(content: string, multipleChunks?: boolean): Promise<void>;
    /**
     * Register a callback to listen for output (stdout and stderr) from the process.
     * @param callback A callback that will be called when the process writes to stdout or stderr with the output as a string.
     */
    onOutput(callback: (data: string) => void): void;
    /**
     * Write data to stdin of the process.
     * @param data The data to write.
     * @returns A promise that resolves when the data has been written.
     */
    write(data: string | Buffer): Promise<void>;
    /**
     * Kill the process.
     * @returns True if the process was killed successfully, false otherwise.
     */
    kill(): boolean;
    /**
     * Check if the process is running.
     * @returns True if the process is running, false otherwise.
     */
    isRunning(): boolean;
    /**
     * Get the exit code of the process. Returns null if the process has not exited yet. Throws an error if the process has not been run yet.
     * @returns The exit code of the process or null.
     */
    getExitCode(): number | null;
    /**
     * Get the output of the process (stdout and stderr).
     * @returns The output of the process as a string.
     */
    getOutput(): string;
    /**
     * Get the next output chunk of the process (stdout and stderr).
     * This must not be a complete line, it can be a partial line, multiple lines or only one carriage return.
     * @returns The next chunk of the process as a string.
     */
    getNextOutput(): Promise<string>;
    /**
     * Get the underlying child process for advanced usage.
     * @returns The underlying child process or undefined if the process has not been run yet.
     * @see https://nodejs.org/api/child_process.html#child_process_class_childprocess
     */
    getChildProcess(): ChildProcessWithoutNullStreams | undefined;
}

export { ansi as ANSI, CLITest, type CLITestOptions };
