var __defProp = Object.defineProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};

// src/ansi.ts
var ansi_exports = {};
__export(ansi_exports, {
  ARROW_DOWN: () => ARROW_DOWN,
  ARROW_LEFT: () => ARROW_LEFT,
  ARROW_RIGHT: () => ARROW_RIGHT,
  ARROW_UP: () => ARROW_UP,
  BACKSPACE: () => BACKSPACE,
  BEL: () => BEL,
  BS: () => BS,
  CR: () => CR,
  CURSOR_BACK: () => CURSOR_BACK,
  CURSOR_DOWN: () => CURSOR_DOWN,
  CURSOR_FORWARD: () => CURSOR_FORWARD,
  CURSOR_HOME: () => CURSOR_HOME,
  CURSOR_HORIZONTAL_ABSOLUTE: () => CURSOR_HORIZONTAL_ABSOLUTE,
  CURSOR_NEXT_LINE: () => CURSOR_NEXT_LINE,
  CURSOR_PREVIOUS_LINE: () => CURSOR_PREVIOUS_LINE,
  CURSOR_UP: () => CURSOR_UP,
  DEL: () => DEL,
  ENTER: () => ENTER,
  ESC: () => ESC,
  FF: () => FF,
  LF: () => LF,
  SPACE: () => SPACE,
  TAB: () => TAB,
  VT: () => VT
});
var CSI = "\x1B[";
var BEL = "\x07";
var BS = "\b";
var TAB = "	";
var LF = "\n";
var VT = "\v";
var FF = "\f";
var CR = "\r";
var ESC = "\x1B";
var DEL = "\x7F";
var SPACE = " ";
var CURSOR_UP = `${CSI}A`;
var CURSOR_DOWN = `${CSI}B`;
var CURSOR_FORWARD = `${CSI}C`;
var CURSOR_BACK = `${CSI}D`;
var CURSOR_NEXT_LINE = `${CSI}E`;
var CURSOR_PREVIOUS_LINE = `${CSI}F`;
var CURSOR_HORIZONTAL_ABSOLUTE = `${CSI}G`;
var CURSOR_HOME = `${CSI}H`;
var ENTER = CR;
var BACKSPACE = BS;
var ARROW_UP = CURSOR_UP;
var ARROW_DOWN = CURSOR_DOWN;
var ARROW_RIGHT = CURSOR_FORWARD;
var ARROW_LEFT = CURSOR_BACK;

// src/cli-test.ts
import { spawn } from "child_process";
import { EventEmitter } from "events";
var genericNotRunErr = (action) => `Can not ${action} when CLI process is not running. Call run() first and ensure the process started and did not exit.`;
var CLITest = class {
  command;
  args;
  childProcess;
  running = false;
  starting = false;
  eventEmitter;
  output = "";
  options;
  /**
   * Create a new CLI test instance.
   * If you want to measure the code coverage of a JS/TS CLI, you may want to prepend the node command with 'c8' or 'nyc'.
   * @param command The command that should be executed to start the CLI process.
   * @param args The arguments that should be passed.
   * @param options Additional options.
   */
  constructor(command, args = [], options = {}) {
    this.command = command;
    this.args = args;
    this.eventEmitter = new EventEmitter();
    const defaultOptions = {
      failOnStderr: true,
      outputEncoding: "utf-8",
      process: {}
    };
    this.options = { ...defaultOptions, ...options };
  }
  /**
   * Execute the CLI process.
   * Tip: You can call this method multiple times to restart the process, after the process has exited.
   * @throws If the process is already running or starting or if the command produces an error.
   * @returns A promise that resolves when the process has been started.
   */
  run() {
    if (this.running || this.starting) {
      throw new Error("Process is already running or starting.");
    }
    this.starting = true;
    this.childProcess = spawn(this.command, this.args, {
      ...this.options.process
    });
    this.childProcess.on("spawn", () => {
      this.running = true;
      this.starting = false;
      this.eventEmitter.emit("spawn");
    });
    this.childProcess.on("exit", (code) => {
      this.running = false;
      this.starting = false;
      this.eventEmitter.emit("exit", code);
    });
    this.childProcess.stdout.on("data", (data) => {
      const dataString = data.toString(this.options.outputEncoding);
      this.output += dataString;
      this.eventEmitter.emit("output", dataString);
    });
    this.childProcess.stderr.on("data", (data) => {
      const dataString = data.toString(this.options.outputEncoding);
      this.output += dataString;
      this.eventEmitter.emit("output", dataString);
      if (this.options.failOnStderr) {
        this.kill();
        throw new Error(
          `Failing because process wrote to stderr. Disable this behavior by setting failOnStderr to false.
${dataString}`
        );
      }
    });
    return new Promise((resolve, reject) => {
      const onSpawn = () => {
        this.eventEmitter.off("error", onError);
        resolve(void 0);
      };
      const onError = (err) => {
        this.eventEmitter.off("spawn", onSpawn);
        reject(`Error starting child process: ${err.message}`);
      };
      this.eventEmitter.once("spawn", onSpawn);
      this.childProcess.once("error", onError);
    });
  }
  /**
   * Wait for the process to exit.
   * If the process returns a non-zero exit code, the promise will not be rejected.
   * @throws If the process is not running or starting.
   * @returns A promise that resolves when the process exits with the exit code.
   */
  waitForExit() {
    if (!this.running && !this.starting) {
      throw new Error(genericNotRunErr("wait for exit"));
    }
    return new Promise((resolve) => {
      this.eventEmitter.once("exit", (code) => {
        resolve(code);
      });
    });
  }
  /**
   * Wait for the process to write the given content to stdout or stderr.
   * If the process exits before the content is found, the promise will be rejected.
   * This method does not check past output, only output that is written after this method is called. For checking past output, use getOutput().
   * Per default this method is only able to find content that is written in a single chunk by the CLI process. If the content is split into multiple chunks set multipleChunks to true.
   *
   * @param content The string to wait for.
   * @param multipleChunks If true, the content can be split into multiple chunks. If false, the content must be written in a single chunk. Defaults to false.
   * @returns A promise that resolves when the content is found, or rejects if the process exits before the content is found.
   */
  waitForOutput(content, multipleChunks = false) {
    if (!this.running && !this.starting) {
      throw new Error(genericNotRunErr("wait for output"));
    }
    if (!content.length) {
      throw new Error("Output to wait for must not be empty.");
    }
    return new Promise((resolve, reject) => {
      if (!multipleChunks) {
        const onExit = () => {
          this.eventEmitter.off("exit", onExit);
          this.eventEmitter.off("output", onOutput);
          reject(new Error(`Process exited before output "${content}" was found`));
        };
        this.eventEmitter.on("exit", onExit);
        const onOutput = (data) => {
          if (data.includes(content)) {
            this.eventEmitter.off("exit", onExit);
            this.eventEmitter.off("output", onOutput);
            return resolve(void 0);
          }
        };
        this.eventEmitter.on("output", onOutput);
      } else {
        const startIndex = this.output.length;
        const onMultiChunkExit = () => {
          this.eventEmitter.off("exit", onMultiChunkExit);
          this.eventEmitter.off("output", onMultiChunkOutput);
          reject(new Error(`Process exited before output "${content}" was found`));
        };
        this.eventEmitter.on("exit", onMultiChunkExit);
        const onMultiChunkOutput = () => {
          const output = this.output.slice(startIndex);
          if (output.includes(content)) {
            this.eventEmitter.off("exit", onMultiChunkExit);
            this.eventEmitter.off("output", onMultiChunkOutput);
            return resolve(void 0);
          }
        };
        this.eventEmitter.on("output", onMultiChunkOutput);
      }
    });
  }
  /**
   * Register a callback to listen for output (stdout and stderr) from the process.
   * @param callback A callback that will be called when the process writes to stdout or stderr with the output as a string.
   */
  onOutput(callback) {
    this.eventEmitter.on("output", (data) => {
      callback(data);
    });
  }
  /**
   * Write data to stdin of the process.
   * @param data The data to write.
   * @returns A promise that resolves when the data has been written.
   */
  write(data) {
    if (!this.running) {
      throw new Error(genericNotRunErr("write data to stdin"));
    }
    return new Promise((resolve, reject) => {
      if (!this.childProcess) {
        return reject(new Error("Child process is undefined"));
      }
      this.childProcess.stdin.write(data, (err) => {
        if (err) {
          return reject(err);
        }
        resolve(void 0);
      });
    });
  }
  /**
   * Kill the process.
   * @returns True if the process was killed successfully, false otherwise.
   */
  kill() {
    if (!this.running) {
      throw new Error(genericNotRunErr("kill process"));
    }
    if (!this.childProcess) {
      throw new Error("Child process is undefined. This should not happen.");
    }
    return this.childProcess.kill();
  }
  /**
   * Check if the process is running.
   * @returns True if the process is running, false otherwise.
   */
  isRunning() {
    return this.running;
  }
  /**
   * Get the exit code of the process. Returns null if the process has not exited yet. Throws an error if the process has not been run yet.
   * @returns The exit code of the process or null.
   */
  getExitCode() {
    if (!this.childProcess) {
      throw new Error(genericNotRunErr("get exit code"));
    }
    return this.childProcess.exitCode;
  }
  /**
   * Get the output of the process (stdout and stderr).
   * @returns The output of the process as a string.
   */
  getOutput() {
    return this.output;
  }
  /**
   * Get the next output chunk of the process (stdout and stderr).
   * This must not be a complete line, it can be a partial line, multiple lines or only one carriage return.
   * @returns The next chunk of the process as a string.
   */
  getNextOutput() {
    if (!this.running && !this.starting) {
      throw new Error(genericNotRunErr("get next output"));
    }
    return new Promise((resolve, reject) => {
      const onExit = () => {
        this.eventEmitter.off("exit", onExit);
        this.eventEmitter.off("output", onOutput);
        reject(new Error("Process exited before next output was found"));
      };
      this.eventEmitter.on("exit", onExit);
      const onOutput = (data) => {
        this.eventEmitter.off("exit", onExit);
        resolve(data);
      };
      this.eventEmitter.once("output", onOutput);
    });
  }
  /**
   * Get the underlying child process for advanced usage.
   * @returns The underlying child process or undefined if the process has not been run yet.
   * @see https://nodejs.org/api/child_process.html#child_process_class_childprocess
   */
  getChildProcess() {
    return this.childProcess;
  }
};
export {
  ansi_exports as ANSI,
  CLITest
};
